package org.vaadin.console;

import java.util.HashSet;
import java.util.Set;

import org.vaadin.console.Console.Command;
import org.vaadin.console.Console.Handler;

/**
 * Default handler for console.
 *
 */
public class DefaultConsoleHandler implements Handler {

    private static final long serialVersionUID = 1L;

    public void handleException(Console console, Exception e, Command cmd,
            String[] argv) {
        e.printStackTrace();
        console.println(e.getClass().getSimpleName() + ": " + e.getMessage());
    }

    public Set<String> getSuggestions(Console console, String input) {

        String prefix = console.parseCommandPrefix(input);
        if (prefix != null) {
            Set<String> matches = new HashSet<String>();
            Set<String> cmds = console.getCommands();
            for (String cmd : cmds) {
                if (cmd.startsWith(prefix)) {
                    matches.add(cmd);
                }
            }
            return matches;
        }
        return null;
    }

    public void inputReceived(Console console, String lastInput) {
        console.parseAndExecuteCommand(lastInput);
        console.prompt();
    }

    public void commandNotFound(Console console, String[] argv) {
        console.print("ERROR: " + argv[0] + ": command not found.");
    }

}
